"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GuardManager = void 0;
const decorator_1 = require("../decorator");
const metadataManager_1 = require("../decorator/metadataManager");
class GuardManager extends Array {
    addGlobalGuard(guards) {
        if (!Array.isArray(guards)) {
            this.push(guards);
        }
        else {
            this.push(...guards);
        }
    }
    async runGuard(ctx, supplierClz, methodName) {
        // check global guard
        for (const Guard of this) {
            const guard = await ctx.requestContext.getAsync(Guard);
            const isPassed = await guard.canActivate(ctx, supplierClz, methodName);
            if (!isPassed) {
                return false;
            }
        }
        // check class Guard
        const classGuardList = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.GUARD_KEY, supplierClz);
        if (classGuardList) {
            for (const Guard of classGuardList) {
                const guard = await ctx.requestContext.getAsync(Guard);
                const isPassed = await guard.canActivate(ctx, supplierClz, methodName);
                if (!isPassed) {
                    return false;
                }
            }
        }
        // check method Guard
        const methodGuardList = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.GUARD_KEY, supplierClz, methodName);
        if (methodGuardList) {
            for (const Guard of methodGuardList) {
                const guard = await ctx.requestContext.getAsync(Guard);
                const isPassed = await guard.canActivate(ctx, supplierClz, methodName);
                if (!isPassed) {
                    return false;
                }
            }
        }
        return true;
    }
}
exports.GuardManager = GuardManager;
//# sourceMappingURL=guardManager.js.map