/// <reference types="node" />
import { ClassType, IIdentifierRelationShip, IMidwayContainer, IMidwayGlobalContainer, IObjectDefinition, IObjectDefinitionRegistry, ObjectIdentifier, ScopeEnum } from '../interface';
import { ManagedResolverFactory } from './managedResolverFactory';
import { EventEmitter } from 'events';
export declare class MidwayContainer implements IMidwayGlobalContainer {
    private _resolverFactory;
    private _registry;
    private _identifierMapping;
    private _objectCreateEventTarget;
    protected ctx: {
        _MAIN_CTX_: boolean;
    };
    private attrMap;
    private namespaceSet;
    private _id;
    get id(): string;
    constructor();
    get objectCreateEventTarget(): EventEmitter<[never]>;
    get registry(): IObjectDefinitionRegistry;
    set registry(registry: IObjectDefinitionRegistry);
    get identifierMapping(): IIdentifierRelationShip;
    addNamespace(ns: string): void;
    bindClass(exports: any, options?: Partial<IObjectDefinition>): void;
    bind<T>(target: T, options?: Partial<IObjectDefinition>): void;
    bind<T>(identifier: ObjectIdentifier, target: T, options?: Partial<IObjectDefinition>): void;
    protected bindModule(module: any, options: Partial<IObjectDefinition>): void;
    setAttr(key: string, value: any): void;
    getAttr<T>(key: string): T;
    getIdentifier(identifier: ClassType | string): string;
    getManagedResolverFactory(): ManagedResolverFactory;
    stop(): Promise<void>;
    get<T>(identifier: ClassType<T> | string, args?: any[]): T;
    getAsync<T>(identifier: ClassType<T> | string, args?: any[]): Promise<T>;
    /**
     * proxy registry.registerObject
     * @param {ObjectIdentifier} identifier
     * @param target
     */
    registerObject(identifier: ObjectIdentifier, target: any): void;
    onBeforeBind(fn: (Clzz: any, options: {
        context: IMidwayContainer;
        definition: IObjectDefinition;
        replaceCallback: (newDefinition: IObjectDefinition) => void;
    }) => void): void;
    onBeforeObjectCreated(fn: (Clzz: any, options: {
        context: IMidwayContainer;
        definition: IObjectDefinition;
        constructorArgs: any[];
    }) => void): void;
    onObjectCreated<T>(fn: (ins: T, options: {
        context: IMidwayContainer;
        definition: IObjectDefinition;
        replaceCallback: (ins: T) => void;
    }) => void): void;
    onObjectInit<T>(fn: (ins: T, options: {
        context: IMidwayContainer;
        definition: IObjectDefinition;
    }) => void): void;
    onBeforeObjectDestroy<T>(fn: (ins: T, options: {
        context: IMidwayContainer;
        definition: IObjectDefinition;
    }) => void): void;
    hasNamespace(ns: string): boolean;
    getNamespaceList(): string[];
    hasDefinition(identifier: ObjectIdentifier): boolean;
    getDefinition(identifier: ObjectIdentifier): IObjectDefinition;
    hasObject(identifier: ObjectIdentifier): boolean;
    getObject<T>(identifier: ObjectIdentifier): T;
    getInstanceScope(instance: any): ScopeEnum | undefined;
}
//# sourceMappingURL=container.d.ts.map