"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectDefinitionRegistry = void 0;
const decorator_1 = require("../decorator");
const PREFIX = '_id_default_';
class LegacyIdentifierRelation extends Map {
    saveClassRelation(module, namespace) {
        const providerId = decorator_1.DecoratorManager.getProviderUUId(module);
        // save uuid
        this.set(providerId, providerId);
        if (providerId) {
            // save alias id
            const aliasId = decorator_1.DecoratorManager.getProviderId(module);
            if (aliasId) {
                // save alias Id
                this.set(aliasId, providerId);
            }
            // save className alias
            this.set(decorator_1.DecoratorManager.getProviderName(module), providerId);
            // save namespace alias
            if (namespace) {
                this.set(namespace + ':' + decorator_1.DecoratorManager.getProviderName(module), providerId);
            }
        }
    }
    saveFunctionRelation(id, uuid) {
        this.set(uuid, uuid);
        this.set(id, uuid);
    }
    hasRelation(id) {
        return this.has(id);
    }
    getRelation(id) {
        return this.get(id);
    }
}
class ObjectDefinitionRegistry extends Map {
    constructor() {
        super(...arguments);
        this.singletonIds = [];
        this._identifierRelation = new LegacyIdentifierRelation();
    }
    get identifierRelation() {
        if (!this._identifierRelation) {
            this._identifierRelation = new LegacyIdentifierRelation();
        }
        return this._identifierRelation;
    }
    set identifierRelation(identifierRelation) {
        this._identifierRelation = identifierRelation;
    }
    get identifiers() {
        const ids = [];
        for (const key of this.keys()) {
            if (key.indexOf(PREFIX) === -1) {
                ids.push(key);
            }
        }
        return ids;
    }
    get count() {
        return this.size;
    }
    getSingletonDefinitionIds() {
        return this.singletonIds;
    }
    getDefinitionByName(name) {
        const definitions = [];
        for (const v of this.values()) {
            const definition = v;
            if (definition.name === name) {
                definitions.push(definition);
            }
        }
        return definitions;
    }
    registerDefinition(identifier, definition) {
        if (definition.isSingletonScope()) {
            this.singletonIds.push(identifier);
        }
        this.set(identifier, definition);
    }
    getDefinition(identifier) {
        identifier = this.identifierRelation.getRelation(identifier) ?? identifier;
        return this.get(identifier);
    }
    removeDefinition(identifier) {
        this.delete(identifier);
    }
    hasDefinition(identifier) {
        identifier = this.identifierRelation.getRelation(identifier) ?? identifier;
        return this.has(identifier);
    }
    clearAll() {
        this.singletonIds = [];
        this.clear();
    }
    hasObject(identifier) {
        identifier = this.identifierRelation.getRelation(identifier) ?? identifier;
        return this.has(PREFIX + identifier);
    }
    registerObject(identifier, target) {
        this.set(PREFIX + identifier, target);
    }
    getObject(identifier) {
        identifier = this.identifierRelation.getRelation(identifier) ?? identifier;
        return this.get(PREFIX + identifier);
    }
    getIdentifierRelation() {
        return this.identifierRelation;
    }
    setIdentifierRelation(identifierRelation) {
        this.identifierRelation = identifierRelation;
    }
}
exports.ObjectDefinitionRegistry = ObjectDefinitionRegistry;
//# sourceMappingURL=definitionRegistry.js.map