"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayRequestContainer = void 0;
const constants_1 = require("../constants");
const definitionRegistry_1 = require("./definitionRegistry");
class MidwayRequestContainer {
    constructor(ctx, applicationContext) {
        this.ctx = ctx;
        this.applicationContext = applicationContext;
        this.registry = new definitionRegistry_1.ObjectDefinitionRegistry();
        this.attrMap = new Map();
        this.parent = applicationContext;
        // update legacy relationship
        this.registry.setIdentifierRelation(this.applicationContext.registry.getIdentifierRelation());
        // register ctx
        this.registerObject(constants_1.REQUEST_CTX_KEY, ctx);
        // register res
        this.registerObject('res', {});
        if (ctx.logger) {
            // register contextLogger
            this.registerObject('logger', ctx.logger);
        }
        ctx[constants_1.REQUEST_CTX_UNIQUE_KEY] = this;
    }
    get(identifier, args) {
        return this.parent
            .getManagedResolverFactory()
            .create(identifier, args, this);
    }
    async getAsync(identifier, args) {
        return this.parent
            .getManagedResolverFactory()
            .createAsync(identifier, args, this);
    }
    getContext() {
        return this.ctx;
    }
    hasDefinition(identifier) {
        return this.parent.hasDefinition(identifier);
    }
    getDefinition(identifier) {
        return this.parent.getDefinition(identifier);
    }
    registerObject(identifier, obj) {
        this.registry.registerObject(identifier, obj);
    }
    hasObject(identifier) {
        return this.registry.hasObject(identifier);
    }
    getObject(identifier) {
        return this.registry.getObject(identifier);
    }
    setAttr(key, value) {
        this.attrMap.set(key, value);
    }
    getAttr(key) {
        return this.attrMap.get(key);
    }
    getIdentifier(identifier) {
        return this.parent.getIdentifier(identifier);
    }
    getInstanceScope(instance) {
        return this.parent.getInstanceScope(instance);
    }
    hasNamespace(namespace) {
        return this.parent.hasNamespace(namespace);
    }
}
exports.MidwayRequestContainer = MidwayRequestContainer;
//# sourceMappingURL=requestContainer.js.map