"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InjectClient = exports.LazyInject = exports.Inject = exports.saveInjectMetadata = void 0;
const decoratorManager_1 = require("../decoratorManager");
const interface_1 = require("../../interface");
const constant_1 = require("../constant");
const types_1 = require("../../util/types");
const metadataManager_1 = require("../metadataManager");
const util_1 = require("../../util");
function getConstructParamNames(target) {
    let paramNames = metadataManager_1.MetadataManager.getOwnMetadata('constructorParamNames', target);
    if (!paramNames) {
        // cache constructor param names
        paramNames = (0, util_1.getParamNames)(target);
        metadataManager_1.MetadataManager.defineMetadata('constructorParamNames', paramNames, target);
    }
    return paramNames;
}
function saveInjectMetadata(identifier, target, targetKey, isLazyInject = false, parameterIndex) {
    // 1、use identifier by user
    // let identifier = opts.identifier;
    let injectMode = interface_1.InjectModeEnum.Identifier;
    let id = identifier;
    const isConstructor = parameterIndex !== undefined;
    // 2、use identifier by class uuid
    if (isConstructor) {
        const paramNames = getConstructParamNames(target);
        if (!id) {
            const argsTypes = metadataManager_1.MetadataManager.getMethodParamTypes(target, targetKey);
            const type = metadataManager_1.MetadataManager.transformTypeFromTSDesign(argsTypes[parameterIndex]);
            if (!type.isBaseType &&
                (0, types_1.isClass)(type.originDesign) &&
                decoratorManager_1.DecoratorManager.isProvide(type.originDesign)) {
                id = decoratorManager_1.DecoratorManager.getProviderUUId(type.originDesign);
                injectMode = interface_1.InjectModeEnum.Class;
            }
            if (!id) {
                // 3、use identifier by property name
                id = paramNames[parameterIndex];
                injectMode = interface_1.InjectModeEnum.SelfName;
            }
        }
        metadataManager_1.MetadataManager.attachMetadata(constant_1.CONSTRUCTOR_INJECT_KEY, {
            targetKey,
            id,
            name: paramNames[parameterIndex],
            injectMode,
            isLazyInject,
            parameterIndex,
        }, target, 'default');
    }
    else {
        if (!id) {
            const type = metadataManager_1.MetadataManager.transformTypeFromTSDesign(metadataManager_1.MetadataManager.getPropertyType(target, targetKey));
            if (!type.isBaseType &&
                (0, types_1.isClass)(type.originDesign) &&
                decoratorManager_1.DecoratorManager.isProvide(type.originDesign)) {
                id = decoratorManager_1.DecoratorManager.getProviderUUId(type.originDesign);
                injectMode = interface_1.InjectModeEnum.Class;
            }
            if (!id) {
                // 3、use identifier by property name
                id = targetKey;
                injectMode = interface_1.InjectModeEnum.SelfName;
            }
        }
        metadataManager_1.MetadataManager.defineMetadata(constant_1.PROPERTY_INJECT_KEY, {
            targetKey,
            id,
            name: targetKey,
            isLazyInject,
            injectMode,
        }, target, targetKey);
    }
}
exports.saveInjectMetadata = saveInjectMetadata;
function Inject(identifier) {
    return function (target, targetKey, parameterIndex) {
        return saveInjectMetadata(identifier, target, targetKey, false, parameterIndex);
    };
}
exports.Inject = Inject;
function LazyInject(identifier) {
    let id = identifier;
    if (id && typeof id !== 'function') {
        id = (() => {
            return identifier;
        });
    }
    return function (target, targetKey, parameterIndex) {
        return saveInjectMetadata(id, target, targetKey, true, parameterIndex);
    };
}
exports.LazyInject = LazyInject;
function InjectClient(serviceFactoryClz, clientName) {
    return decoratorManager_1.DecoratorManager.createCustomPropertyDecorator(constant_1.FACTORY_SERVICE_CLIENT_KEY, {
        serviceFactoryClz,
        clientName,
    });
}
exports.InjectClient = InjectClient;
//# sourceMappingURL=inject.js.map