"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineConfiguration = exports.FunctionalConfiguration = void 0;
const decorator_1 = require("../decorator");
const metadataManager_1 = require("../decorator/metadataManager");
class FunctionalConfiguration {
    constructor(options) {
        this.options = options;
    }
    async onConfigLoad(container, mainApp) {
        return this.options?.onConfigLoad?.(container, mainApp);
    }
    async onReady(container, mainApp) {
        return this.options?.onReady?.(container, mainApp);
    }
    async onServerReady(container, mainApp) {
        return this.options?.onServerReady?.(container, mainApp);
    }
    async onHealthCheck(container) {
        return this.options?.onHealthCheck?.(container);
    }
    async onStop(container, mainApp) {
        return this.options?.onStop?.(container, mainApp);
    }
    // object lifecycle
    onBeforeObjectCreated(Clzz, options) {
        return this.options?.onBeforeObjectCreated?.(Clzz, options);
    }
    onObjectCreated(ins, options) {
        return this.options?.onObjectCreated?.(ins, options);
    }
    onObjectInit(ins, options) {
        return this.options?.onObjectInit?.(ins, options);
    }
    onBeforeObjectDestroy(ins, options) {
        return this.options?.onBeforeObjectDestroy?.(ins, options);
    }
}
exports.FunctionalConfiguration = FunctionalConfiguration;
const defineConfiguration = (options) => {
    const configuration = new FunctionalConfiguration(options);
    metadataManager_1.MetadataManager.ensureTargetType(configuration, metadataManager_1.MetadataManager.ObjectType.Object);
    metadataManager_1.MetadataManager.defineMetadata(decorator_1.CONFIGURATION_OBJECT_KEY, options, configuration);
    return configuration;
};
exports.defineConfiguration = defineConfiguration;
//# sourceMappingURL=configuration.js.map