"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMainApp = exports.useApp = exports.useConfig = exports.useInjectSync = exports.useInject = exports.usePlugin = exports.useLogger = exports.useContext = void 0;
const contextUtil_1 = require("../util/contextUtil");
const constants_1 = require("../constants");
const applicationManager_1 = require("../common/applicationManager");
const configService_1 = require("../service/configService");
function useContext() {
    const ctx = (0, contextUtil_1.getCurrentAsyncContextManager)()
        .active()
        .getValue(constants_1.ASYNC_CONTEXT_KEY);
    return ctx;
}
exports.useContext = useContext;
function useLogger(loggerName) {
    const ctx = useContext();
    if (ctx) {
        if (loggerName) {
            return ctx.logger.getLogger(loggerName);
        }
        return ctx.logger;
    }
    else {
        return useMainApp().getLogger(loggerName);
    }
}
exports.useLogger = useLogger;
function usePlugin(key) {
    const ctx = useContext();
    return ctx ? ctx.app[key] || ctx[key] : useMainApp()[key];
}
exports.usePlugin = usePlugin;
async function useInject(identifier, args) {
    const ctx = useContext();
    const requestContext = ctx
        ? ctx['requestContext']
        : useMainApp().getApplicationContext();
    return requestContext.getAsync(identifier, args);
}
exports.useInject = useInject;
function useInjectSync(identifier, args) {
    ``;
    const ctx = useContext();
    const requestContext = ctx
        ? ctx['requestContext']
        : useMainApp().getApplicationContext();
    return requestContext.get(identifier, args);
}
exports.useInjectSync = useInjectSync;
function useConfig(key) {
    return useMainApp()
        .getApplicationContext()
        .get(configService_1.MidwayConfigService)
        .getConfiguration(key);
}
exports.useConfig = useConfig;
function useApp(appName) {
    const applicationManager = useMainApp()
        .getApplicationContext()
        .get(applicationManager_1.MidwayApplicationManager);
    return applicationManager.getApplication(appName);
}
exports.useApp = useApp;
function useMainApp() {
    return (0, contextUtil_1.getCurrentMainApp)();
}
exports.useMainApp = useMainApp;
//# sourceMappingURL=hooks.js.map