"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getObjectDefinition = exports.saveObjectDefinition = exports.getPropertyInject = exports.savePropertyInject = exports.getMethodParamTypes = exports.getPropertyType = exports.createCustomParamDecorator = exports.createCustomMethodDecorator = exports.createCustomPropertyDecorator = exports.isProvide = exports.getProviderUUId = exports.getProviderName = exports.getProviderId = exports.saveProviderId = exports.clearAllModule = exports.resetModule = exports.listModule = exports.saveModule = exports.listPreloadModule = exports.savePreloadModule = exports.getPropertyMetadata = exports.attachPropertyMetadata = exports.savePropertyMetadata = exports.listPropertyDataFromClass = exports.getPropertyDataFromClass = exports.attachPropertyDataToClass = exports.savePropertyDataToClass = exports.getClassMetadata = exports.getClassExtendedMetadata = exports.attachClassMetadata = exports.saveClassMetadata = void 0;
const metadataManager_1 = require("../decorator/metadataManager");
const decorator_1 = require("../decorator");
const inject_1 = require("../decorator/common/inject");
const constants_1 = require("./constants");
/**
 * the key for attach and list property data from class
 */
const legacyListPropertyDataKey = 'LEGACY_LIST_PROPERTY_DATA_KEY';
function getDecoratorClsMethodPrefix(decoratorNameKey) {
    return decoratorNameKey.toString() + '_CLS_METHOD';
}
function getDecoratorClsMethodKey(decoratorNameKey, methodKey) {
    return (getDecoratorClsMethodPrefix(decoratorNameKey) + ':' + methodKey.toString());
}
/**
 * save data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param mergeIfExist
 * @since 2.3.0
 * @deprecated Use MetadataManager.defineMetadata instead
 */
function saveClassMetadata(decoratorNameKey, data, target, mergeIfExist) {
    if (mergeIfExist && typeof data === 'object') {
        const originData = metadataManager_1.MetadataManager.getMetadata(decoratorNameKey, target);
        if (!originData) {
            return metadataManager_1.MetadataManager.defineMetadata(decoratorNameKey, data, target);
        }
        if (Array.isArray(originData)) {
            return metadataManager_1.MetadataManager.defineMetadata(decoratorNameKey, originData.concat(data), target);
        }
        else {
            return metadataManager_1.MetadataManager.defineMetadata(decoratorNameKey, Object.assign(originData, data), target);
        }
    }
    return metadataManager_1.MetadataManager.defineMetadata(decoratorNameKey, data, target);
}
exports.saveClassMetadata = saveClassMetadata;
/**
 * attach data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param groupBy
 * @since 2.3.0
 * @deprecated Use MetadataManager.attachMetadata instead
 */
function attachClassMetadata(decoratorNameKey, data, target, groupBy) {
    return metadataManager_1.MetadataManager.attachMetadata(decoratorNameKey, data, target, groupBy);
}
exports.attachClassMetadata = attachClassMetadata;
function _getClassExtendedMetadata(decoratorNameKey, target, propertyName, useCache) {
    const ret = metadataManager_1.MetadataManager.getPropertiesWithMetadata(decoratorNameKey, target);
    // array item to object
    const res = {};
    for (const key in ret) {
        const element = ret[key];
        if (Array.isArray(element) && element.length) {
            res[key] = element[element.length - 1];
        }
        else {
            res[key] = element;
        }
    }
    return res;
}
/**
 * get data from class and proto
 * @param decoratorNameKey
 * @param target
 * @param propertyName
 * @param useCache
 * @since 2.3.0
 * @deprecated Use MetadataManager.getMetadata instead
 */
function getClassExtendedMetadata(decoratorNameKey, target, propertyName, useCache) {
    const ret = metadataManager_1.MetadataManager.getMetadata(decoratorNameKey, target);
    if (ret === undefined) {
        const res = _getClassExtendedMetadata(decoratorNameKey, target, propertyName, useCache);
        if (res) {
            return res;
        }
    }
    return ret;
}
exports.getClassExtendedMetadata = getClassExtendedMetadata;
/**
 * get data from class
 * @param decoratorNameKey
 * @param target
 * @since 2.3.0
 * @deprecated Use MetadataManager.getOwnMetadata instead
 */
function getClassMetadata(decoratorNameKey, target) {
    return metadataManager_1.MetadataManager.getOwnMetadata(decoratorNameKey, target);
}
exports.getClassMetadata = getClassMetadata;
/**
 * save property data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated Use MetadataManager.defineMetadata instead
 */
function savePropertyDataToClass(decoratorNameKey, data, target, propertyName) {
    const dataKey = getDecoratorClsMethodKey(decoratorNameKey, propertyName);
    const originMap = metadataManager_1.MetadataManager.getOwnMetadata(legacyListPropertyDataKey, target) ||
        new Map();
    originMap.set(dataKey, data);
    return metadataManager_1.MetadataManager.defineMetadata(legacyListPropertyDataKey, originMap, target);
}
exports.savePropertyDataToClass = savePropertyDataToClass;
/**
 * attach property data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated
 */
function attachPropertyDataToClass(decoratorNameKey, data, target, propertyName) {
    const dataKey = getDecoratorClsMethodKey(decoratorNameKey, propertyName);
    const originMap = metadataManager_1.MetadataManager.getOwnMetadata(legacyListPropertyDataKey, target) ||
        new Map();
    if (originMap.has(dataKey)) {
        const ret = originMap.get(dataKey);
        ret.push(data);
        originMap.set(dataKey, ret);
    }
    else {
        originMap.set(dataKey, [data]);
    }
    return metadataManager_1.MetadataManager.defineMetadata(legacyListPropertyDataKey, originMap, target);
}
exports.attachPropertyDataToClass = attachPropertyDataToClass;
/**
 * get property data from class
 * @param decoratorNameKey
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated Use MetadataManager.getOwnMetadata instead
 */
function getPropertyDataFromClass(decoratorNameKey, target, propertyName) {
    const dataKey = getDecoratorClsMethodKey(decoratorNameKey, propertyName);
    const originMap = metadataManager_1.MetadataManager.getOwnMetadata(legacyListPropertyDataKey, target);
    return originMap && originMap.get(dataKey);
}
exports.getPropertyDataFromClass = getPropertyDataFromClass;
/**
 * list property data from class
 * @param decoratorNameKey
 * @param target
 * @since 2.3.0
 * @deprecated
 */
function listPropertyDataFromClass(decoratorNameKey, target) {
    const originMap = metadataManager_1.MetadataManager.getOwnMetadata(legacyListPropertyDataKey, target);
    const res = [];
    const prefix = getDecoratorClsMethodPrefix(decoratorNameKey);
    for (const [key, value] of originMap) {
        if (key.indexOf(prefix) !== -1) {
            res.push(value);
        }
    }
    return res;
}
exports.listPropertyDataFromClass = listPropertyDataFromClass;
/**
 * save property data
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated Use MetadataManager.defineMetadata instead
 */
function savePropertyMetadata(decoratorNameKey, data, target, propertyName) {
    return metadataManager_1.MetadataManager.defineMetadata(decoratorNameKey, data, target, propertyName);
}
exports.savePropertyMetadata = savePropertyMetadata;
/**
 * attach property data
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated
 */
function attachPropertyMetadata(decoratorNameKey, data, target, propertyName) {
    return metadataManager_1.MetadataManager.attachMetadata(decoratorNameKey, data, target, propertyName);
}
exports.attachPropertyMetadata = attachPropertyMetadata;
/**
 * get property data
 * @param decoratorNameKey
 * @param target
 * @param propertyName
 * @since 2.3.0
 * @deprecated Use MetadataManager.getOwnMetadata instead
 */
function getPropertyMetadata(decoratorNameKey, target, propertyName) {
    return metadataManager_1.MetadataManager.getOwnMetadata(decoratorNameKey, target, propertyName);
}
exports.getPropertyMetadata = getPropertyMetadata;
/**
 * save preload module by target
 * @param target
 * @since 2.0.0
 * @deprecated Use DecoratorManager.savePreStartModule instead
 */
function savePreloadModule(target) {
    return decorator_1.DecoratorManager.savePreStartModule(target);
}
exports.savePreloadModule = savePreloadModule;
/**
 * list preload module
 * @since 2.0.0
 * @deprecated Use DecoratorManager.listPreStartModule instead
 */
function listPreloadModule() {
    return decorator_1.DecoratorManager.listPreStartModule();
}
exports.listPreloadModule = listPreloadModule;
/**
 * save module to inner map
 * @param decoratorNameKey
 * @param target
 * @since 2.0.0
 * @deprecated Use DecoratorManager.saveModule instead
 */
function saveModule(decoratorNameKey, target) {
    return decorator_1.DecoratorManager.saveModule(decoratorNameKey, target);
}
exports.saveModule = saveModule;
/**
 * list module from decorator key
 * @param decoratorNameKey
 * @param filter
 * @since 2.0.0
 * @deprecated Use DecoratorManager.listModule instead
 */
function listModule(decoratorNameKey, filter) {
    return decorator_1.DecoratorManager.listModule(decoratorNameKey, filter);
}
exports.listModule = listModule;
/**
 * reset module
 * @param decoratorNameKey
 * @since 2.0.0
 * @deprecated Use DecoratorManager.resetModule instead
 */
function resetModule(decoratorNameKey) {
    return decorator_1.DecoratorManager.resetModule(decoratorNameKey);
}
exports.resetModule = resetModule;
/**
 * clear all module
 * @since 3.0.0
 * @deprecated Use DecoratorManager.clearAllModule instead
 */
function clearAllModule() {
    return decorator_1.DecoratorManager.clearAllModule();
}
exports.clearAllModule = clearAllModule;
/**
 * class provider id
 * @since 2.3.0
 * @deprecated Use DecoratorManager.saveProviderId instead
 */
function saveProviderId(identifier, target) {
    return decorator_1.DecoratorManager.saveProviderId(identifier, target);
}
exports.saveProviderId = saveProviderId;
/**
 * get provider id from module
 * @since 3.0.0
 * @deprecated Use DecoratorManager.getProviderId instead
 */
function getProviderId(module) {
    return decorator_1.DecoratorManager.getProviderId(module);
}
exports.getProviderId = getProviderId;
/**
 * @since 3.0.0
 * @deprecated Use DecoratorManager.getProviderName instead
 */
function getProviderName(module) {
    return decorator_1.DecoratorManager.getProviderName(module);
}
exports.getProviderName = getProviderName;
/**
 * get provider uuid from module
 * @since 3.0.0
 * @deprecated Use DecoratorManager.getProviderUUId instead
 */
function getProviderUUId(module) {
    return decorator_1.DecoratorManager.getProviderUUId(module);
}
exports.getProviderUUId = getProviderUUId;
/**
 * use @Provide decorator or not
 * @since 3.0.0
 * @deprecated Use DecoratorManager.isProvide instead
 */
function isProvide(target) {
    return decorator_1.DecoratorManager.isProvide(target);
}
exports.isProvide = isProvide;
/**
 * Create a custom property inject
 * @param decoratorKey
 * @param metadata
 * @param impl default true, configuration need decoratorService.registerMethodHandler
 * @since 3.0.0
 * @deprecated Use DecoratorManager.createCustomPropertyDecorator instead
 */
function createCustomPropertyDecorator(decoratorKey, metadata, impl = true) {
    return decorator_1.DecoratorManager.createCustomPropertyDecorator(decoratorKey, metadata, impl);
}
exports.createCustomPropertyDecorator = createCustomPropertyDecorator;
/**
 * Create a custom method decorator
 * @param decoratorKey
 * @param metadata
 * @param implOrOptions
 * @since 3.0.0
 * @deprecated Use DecoratorManager.createCustomMethodDecorator instead
 */
function createCustomMethodDecorator(decoratorKey, metadata, implOrOptions = { impl: true }) {
    return decorator_1.DecoratorManager.createCustomMethodDecorator(decoratorKey, metadata, implOrOptions);
}
exports.createCustomMethodDecorator = createCustomMethodDecorator;
/**
 * Create a custom param decorator
 * @param decoratorKey
 * @param metadata
 * @param implOrOptions
 * @since 3.0.0
 * @deprecated Use DecoratorManager.createCustomParamDecorator instead
 */
function createCustomParamDecorator(decoratorKey, metadata, implOrOptions = { impl: true }) {
    return decorator_1.DecoratorManager.createCustomParamDecorator(decoratorKey, metadata, implOrOptions);
}
exports.createCustomParamDecorator = createCustomParamDecorator;
/**
 * get property(method) type from metadata
 * @param target
 * @param methodName
 * @since 3.0.0
 * @deprecated Use MetadataManager.getPropertyType instead
 */
function getPropertyType(target, methodName) {
    return metadataManager_1.MetadataManager.transformTypeFromTSDesign(metadataManager_1.MetadataManager.getPropertyType(target, methodName));
}
exports.getPropertyType = getPropertyType;
/**
 * get parameters type by reflect-metadata
 * @since 3.0.0
 * @deprecated Use MetadataManager.getMethodParamTypes instead
 */
function getMethodParamTypes(target, methodName) {
    return metadataManager_1.MetadataManager.getMethodParamTypes(target, methodName);
}
exports.getMethodParamTypes = getMethodParamTypes;
/**
 * save property inject args
 * @param opts 参数
 * @since 2.3.0
 * @deprecated Use MetadataManager.attachMetadata instead
 */
function savePropertyInject(opts) {
    (0, inject_1.saveInjectMetadata)(opts.identifier, opts.target, opts.targetKey);
}
exports.savePropertyInject = savePropertyInject;
/**
 * get property inject args
 * @param target
 * @param useCache
 * @since 2.3.0
 * @deprecated Use MetadataManager.getMetadata instead
 */
function getPropertyInject(target, useCache) {
    const ret = _getClassExtendedMetadata(decorator_1.PROPERTY_INJECT_KEY, target, undefined, useCache);
    for (const key in ret) {
        const element = ret[key];
        if (Array.isArray(element) && element.length) {
            ret[key] = element[element.length - 1];
        }
    }
    return ret;
}
exports.getPropertyInject = getPropertyInject;
/**
 * save class object definition
 * @param target class
 * @param props property data
 * @since 2.3.0
 * @deprecated Use MetadataManager.attachMetadata instead
 */
function saveObjectDefinition(target, props = {}) {
    metadataManager_1.MetadataManager.attachMetadata(constants_1.OBJ_DEF_CLS, props, target, '__fake_object_def_method');
    return target;
}
exports.saveObjectDefinition = saveObjectDefinition;
/**
 * get class object definition from metadata
 * @param target
 * @since 2.3.0
 * @deprecated Use MetadataManager.getPropertiesWithMetadata instead
 */
function getObjectDefinition(target) {
    /**
     * Array(1) [{…}]
     */
    const ret = _getClassExtendedMetadata(constants_1.OBJ_DEF_CLS, target);
    const scope = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.SCOPE_KEY, target);
    if (Array.isArray(ret)) {
        const res = {};
        for (const v of ret) {
            Object.assign(res, v);
        }
        return res;
    }
    else {
        // {
        //   "abcde": [
        //     {
        //       "initMethod": "abcde"
        //     }
        //   ],
        //   "destroy": [
        //     {
        //       "destroyMethod": "destroy"
        //     }
        //   ]
        // }
        // merge object and get
        // {
        //   "destroyMethod": "destroy",
        //   "initMethod": "abcde",
        // }
        const res = {};
        for (const key in ret) {
            const element = ret[key];
            if (Array.isArray(element)) {
                for (const v of element) {
                    Object.assign(res, v);
                }
            }
            else {
                Object.assign(res, element);
            }
        }
        // merge scope
        return Object.assign(res, scope);
    }
}
exports.getObjectDefinition = getObjectDefinition;
//# sourceMappingURL=decorator.js.map