"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayServerlessFunctionService = void 0;
const decorator_1 = require("../decorator");
const webRouterService_1 = require("./webRouterService");
const interface_1 = require("../interface");
const metadataManager_1 = require("../decorator/metadataManager");
let MidwayServerlessFunctionService = class MidwayServerlessFunctionService extends webRouterService_1.MidwayWebRouterService {
    constructor(options = {}) {
        super(Object.assign({}, options, {
            includeFunctionRouter: true,
        }));
        this.options = options;
    }
    async analyze() {
        this.analyzeController();
        this.analyzeFunction();
        this.sortPrefixAndRouter();
        // requestMethod all transform to other method
        for (const routerInfo of this.routes.values()) {
            for (const info of routerInfo) {
                if (info.requestMethod === 'all') {
                    info.functionTriggerMetadata = info.functionTriggerMetadata || {};
                    info.functionTriggerMetadata.method = [
                        'get',
                        'post',
                        'put',
                        'delete',
                        'head',
                        'patch',
                        'options',
                    ];
                }
            }
        }
    }
    analyzeFunction() {
        const fnModules = decorator_1.DecoratorManager.listModule(decorator_1.FUNC_KEY);
        for (const module of fnModules) {
            this.collectFunctionRoute(module);
        }
    }
    collectFunctionRoute(module) {
        // serverlessTrigger metadata
        const webRouterInfo = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.FUNC_KEY, module);
        const controllerId = decorator_1.DecoratorManager.getProviderName(module);
        const id = decorator_1.DecoratorManager.getProviderUUId(module);
        const prefix = '/';
        if (!this.routes.has(prefix)) {
            this.routes.set(prefix, []);
            this.routesPriority.push({
                prefix,
                priority: -999,
                middleware: [],
                routerOptions: {},
                controllerId,
                routerModule: module,
            });
        }
        for (const webRouter of webRouterInfo) {
            // 新的 @ServerlessTrigger 写法
            if (webRouter['metadata']?.['path']) {
                const routeArgsInfo = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.WEB_ROUTER_PARAM_KEY, module, webRouter['methodName']) || [];
                const routerResponseData = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.WEB_RESPONSE_KEY, module, webRouter['methodName']) || [];
                // 新 http/api gateway 函数
                const data = {
                    id,
                    prefix,
                    routerName: '',
                    url: webRouter['metadata']['path'],
                    requestMethod: webRouter['metadata']?.['method'] ?? 'get',
                    method: webRouter['methodName'],
                    description: '',
                    summary: '',
                    handlerName: `${controllerId}.${webRouter['methodName']}`,
                    funcHandlerName: `${controllerId}.${webRouter['methodName']}`,
                    controllerId,
                    middleware: webRouter['metadata']?.['middleware'] || [],
                    controllerMiddleware: [],
                    requestMetadata: routeArgsInfo,
                    responseMetadata: routerResponseData,
                };
                const functionMeta = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.SERVERLESS_FUNC_KEY, module, webRouter['methodName']) || {};
                const functionName = functionMeta['functionName'] ??
                    webRouter?.['metadata']?.['functionName'] ??
                    createFunctionName(module, webRouter['methodName']);
                const funcHandlerName = functionMeta['handlerName'] ??
                    webRouter?.['metadata']?.['handlerName'] ??
                    data.funcHandlerName;
                data.functionName = functionName;
                data.funcHandlerName = funcHandlerName;
                data.functionTriggerName = webRouter['type'];
                data.functionTriggerMetadata = webRouter['metadata'];
                data.functionMetadata = {
                    functionName,
                    ...functionMeta,
                };
                this.checkDuplicateAndPush(prefix, data);
            }
            else {
                const functionMeta = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.SERVERLESS_FUNC_KEY, module, webRouter['methodName']) || {};
                const functionName = functionMeta['functionName'] ??
                    webRouter?.['metadata']?.['functionName'] ??
                    createFunctionName(module, webRouter['methodName']);
                const funcHandlerName = functionMeta['handlerName'] ??
                    webRouter?.['metadata']?.['handlerName'] ??
                    `${controllerId}.${webRouter['methodName']}`;
                // 其他类型的函数
                this.checkDuplicateAndPush(prefix, {
                    id,
                    prefix,
                    routerName: '',
                    url: '',
                    requestMethod: '',
                    method: webRouter['methodName'],
                    description: '',
                    summary: '',
                    handlerName: `${controllerId}.${webRouter['methodName']}`,
                    funcHandlerName: funcHandlerName,
                    controllerId,
                    middleware: webRouter['metadata']?.['middleware'] || [],
                    controllerMiddleware: [],
                    requestMetadata: [],
                    responseMetadata: [],
                    functionName,
                    functionTriggerName: webRouter['type'],
                    functionTriggerMetadata: webRouter['metadata'],
                    functionMetadata: {
                        functionName,
                        ...functionMeta,
                    },
                });
            }
        }
    }
    async getFunctionList() {
        return this.getFlattenRouterTable({
            compileUrlPattern: true,
        });
    }
    addServerlessFunction(func, triggerOptions, functionOptions = {}) {
        const prefix = '';
        if (!this.routes.has(prefix)) {
            this.routes.set(prefix, []);
            this.routesPriority.push({
                prefix,
                priority: 0,
                middleware: [],
                routerOptions: {},
                controllerId: undefined,
                routerModule: undefined,
            });
        }
        const functionName = triggerOptions.functionName ?? functionOptions.functionName;
        this.checkDuplicateAndPush(prefix, {
            id: null,
            method: func,
            url: triggerOptions.metadata['path'] || '',
            requestMethod: triggerOptions.metadata['method'] || '',
            description: '',
            summary: '',
            handlerName: '',
            funcHandlerName: triggerOptions.handlerName || functionOptions.handlerName,
            controllerId: '',
            middleware: triggerOptions.metadata?.middleware || [],
            controllerMiddleware: [],
            requestMetadata: [],
            responseMetadata: [],
            functionName,
            functionTriggerName: triggerOptions.metadata.name,
            functionTriggerMetadata: triggerOptions.metadata,
            functionMetadata: {
                functionName,
                ...functionOptions,
            },
        });
    }
};
exports.MidwayServerlessFunctionService = MidwayServerlessFunctionService;
exports.MidwayServerlessFunctionService = MidwayServerlessFunctionService = __decorate([
    (0, decorator_1.Provide)(),
    (0, decorator_1.Scope)(interface_1.ScopeEnum.Singleton),
    __metadata("design:paramtypes", [Object])
], MidwayServerlessFunctionService);
function createFunctionName(target, functionName) {
    return (decorator_1.DecoratorManager.getProviderName(target).replace(/[:#]/g, '-') +
        '-' +
        functionName);
}
//# sourceMappingURL=slsFunctionService.js.map