"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathFileUtil = exports.getModuleRequirePathList = exports.normalizePath = exports.getFileContentSync = exports.isPathEqual = exports.isPath = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
function isPath(p) {
    // eslint-disable-next-line no-useless-escape
    if (/(^[\.\/])|:|\\/.test(p)) {
        return true;
    }
    return false;
}
exports.isPath = isPath;
function isPathEqual(one, two) {
    if (!one || !two) {
        return false;
    }
    const ext = (0, path_1.extname)(one);
    return one.replace(ext, '') === two;
}
exports.isPathEqual = isPathEqual;
function getFileContentSync(filePath, encoding) {
    return typeof filePath === 'string'
        ? (0, fs_1.readFileSync)(filePath, {
            encoding,
        })
        : filePath;
}
exports.getFileContentSync = getFileContentSync;
/**
 * Normalize path, if p is absolute path, return p, otherwise join p with baseDir
 *
 * @example
 * ```ts
 * normalizePath('/a/b', 'c') => '/a/b/c'
 * normalizePath('/a/b', '/c') => '/c'
 * ```
 *
 * @param baseDir
 * @param p
 */
function normalizePath(baseDir, p) {
    if ((0, path_1.isAbsolute)(p)) {
        return p;
    }
    else {
        return (0, path_1.resolve)(baseDir, p);
    }
}
exports.normalizePath = normalizePath;
function getModuleRequirePathList(moduleName) {
    const moduleNameList = [moduleName, moduleName.replace(/\//g, '_')];
    let moduleNameMap = {};
    const modulePathList = [];
    Object.keys(require.cache || {}).forEach(moduleName => {
        let moduleIndex = -1;
        for (const moduleName of moduleNameList) {
            const index = moduleName.indexOf(moduleName);
            if (index !== -1) {
                moduleIndex = index;
                break;
            }
        }
        if (moduleIndex === -1) {
            return;
        }
        const modulePath = moduleName.slice(0, moduleIndex);
        if (moduleNameMap[modulePath]) {
            return;
        }
        moduleNameMap[modulePath] = true;
        modulePathList.push(modulePath);
    });
    moduleNameMap = undefined;
    return modulePathList;
}
exports.getModuleRequirePathList = getModuleRequirePathList;
exports.PathFileUtil = {
    isPath,
    isPathEqual,
    getFileContentSync,
};
//# sourceMappingURL=pathFileUtil.js.map